@@ -80,16 +80,54 @@ else:
 
 
 class Embedding(keras.layers.Embedding):
-    """为了适配T5，对Embedding的Mask做特殊处理
+    """拓展Embedding层
     """
     def compute_mask(self, inputs, mask=None):
-        """保证第一个token不被mask
+        """为了适配T5，保证第一个token不被mask
         """
-        mask = super(Embedding, self).compute_mask(inputs, mask)
-        if mask is not None:
-            mask1 = K.ones_like(mask[:, :1], dtype='bool')
-            mask2 = mask[:, 1:]
-            return K.concatenate([mask1, mask2], 1)
+        if self._current_mode == 'embedding':
+            mask = super(Embedding, self).compute_mask(inputs, mask)
+            if mask is not None:
+                mask1 = K.ones_like(mask[:, :1], dtype='bool')
+                mask2 = mask[:, 1:]
+                return K.concatenate([mask1, mask2], 1)
+        else:
+            return mask
+
+    def call(self, inputs, mode='embedding'):
+        """新增mode参数，可以为embedding或dense。如果为embedding，
+        则等价于普通Embedding层；如果为dense，则等价于无bias的Dense层。
+        """
+        self._current_mode = mode
+        if mode == 'embedding':
+            return super(Embedding, self).call(inputs)
+        else:
+            kernel = K.transpose(self.embeddings)
+            return K.dot(inputs, kernel)
+
+    def compute_output_shape(self, input_shape):
+        if self._current_mode == 'embedding':
+            return super(Embedding, self).compute_output_shape(input_shape)
+        else:
+            return input_shape[:2] + (K.int_shape(self.embeddings)[0],)
+
+
+class BiasAdd(Layer):
+    """加上偏置项
+    """
+    @integerize_shape
+    def build(self, input_shape):
+        super(BiasAdd, self).build(input_shape)
+        output_dim = input_shape[-1]
+        self.bias = self.add_weight(
+            name='bias',
+            shape=(output_dim,),
+            initializer='zeros',
+            trainable=True
+        )
+
+    def call(self, inputs):
+        return K.bias_add(inputs, self.bias)
 
 
 class MultiHeadAttention(Layer):
@@ -305,6 +343,12 @@ class LayerNormalization(Layer):
 
         return outputs
 
+    def compute_output_shape(self, input_shape):
+        if self.conditional:
+            return input_shape[0]
+        else:
+            return input_shape
+
     def get_config(self):
         config = {
             'center': self.center,
@@ -329,6 +373,7 @@ class PositionEmbedding(Layer):
         output_dim,
         merge_mode='add',
         embeddings_initializer='zeros',
+        custom_position_ids=False,
         **kwargs
     ):
         super(PositionEmbedding, self).__init__(**kwargs)
@@ -336,6 +381,7 @@ class PositionEmbedding(Layer):
         self.output_dim = output_dim
         self.merge_mode = merge_mode
         self.embeddings_initializer = initializers.get(embeddings_initializer)
+        self.custom_position_ids = custom_position_ids
 
     def build(self, input_shape):
         super(PositionEmbedding, self).build(input_shape)
@@ -346,18 +392,30 @@ class PositionEmbedding(Layer):
         )
 
     def call(self, inputs):
-        input_shape = K.shape(inputs)
-        batch_size, seq_len = input_shape[0], input_shape[1]
-        pos_embeddings = self.embeddings[:seq_len]
-        pos_embeddings = K.expand_dims(pos_embeddings, 0)
+        """如果custom_position_ids，那么第二个输入为自定义的位置id
+        """
+        if self.custom_position_ids:
+            inputs, position_ids = inputs
+            if K.dtype(position_ids) != 'int32':
+                position_ids = K.cast(position_ids, 'int32')
+            pos_embeddings = K.gather(self.embeddings, position_ids)
+        else:
+            input_shape = K.shape(inputs)
+            batch_size, seq_len = input_shape[0], input_shape[1]
+            pos_embeddings = self.embeddings[:seq_len]
+            pos_embeddings = K.expand_dims(pos_embeddings, 0)
+            if self.merge_mode != 'add':
+                pos_embeddings = K.tile(pos_embeddings, [batch_size, 1, 1])
 
         if self.merge_mode == 'add':
             return inputs + pos_embeddings
         else:
-            pos_embeddings = K.tile(pos_embeddings, [batch_size, 1, 1])
             return K.concatenate([inputs, pos_embeddings])
 
     def compute_output_shape(self, input_shape):
+        if self.custom_position_ids:
+            input_shape = input_shape[0]
+
         if self.merge_mode == 'add':
             return input_shape
         else:
@@ -370,6 +428,7 @@ class PositionEmbedding(Layer):
             'merge_mode': self.merge_mode,
             'embeddings_initializer':
                 initializers.serialize(self.embeddings_initializer),
+            'custom_position_ids': self.custom_position_ids,
         }
         base_config = super(PositionEmbedding, self).get_config()
         return dict(list(base_config.items()) + list(config.items()))
@@ -541,50 +600,6 @@ class FeedForward(Layer):
         return dict(list(base_config.items()) + list(config.items()))
 
 
-class EmbeddingDense(Layer):
-    """运算跟Dense一致，但kernel用Embedding层的embeddings矩阵。
-    根据Embedding层的名字来搜索定位Embedding层。
-    """
-    def __init__(
-        self, embedding_name, activation='softmax', use_bias=True, **kwargs
-    ):
-        super(EmbeddingDense, self).__init__(**kwargs)
-        self.embedding_name = embedding_name
-        self.activation = activations.get(activation)
-        self.use_bias = use_bias
-
-    def call(self, inputs):
-        if not hasattr(self, 'kernel'):
-            embedding_layer = search_layer(inputs, self.embedding_name)
-            if embedding_layer is None:
-                raise Exception('Embedding layer not found')
-
-            self.kernel = K.transpose(embedding_layer.embeddings)
-            self.units = K.int_shape(self.kernel)[1]
-            if self.use_bias:
-                self.bias = self.add_weight(
-                    name='bias', shape=(self.units,), initializer='zeros'
-                )
-
-        outputs = K.dot(inputs, self.kernel)
-        if self.use_bias:
-            outputs = K.bias_add(outputs, self.bias)
-        outputs = self.activation(outputs)
-        return outputs
-
-    def compute_output_shape(self, input_shape):
-        return input_shape[:-1] + (self.units,)
-
-    def get_config(self):
-        config = {
-            'embedding_name': self.embedding_name,
-            'activation': activations.serialize(self.activation),
-            'use_bias': self.use_bias,
-        }
-        base_config = super(EmbeddingDense, self).get_config()
-        return dict(list(base_config.items()) + list(config.items()))
-
-
 class ConditionalRandomField(Layer):
     """纯Keras实现CRF层
     CRF层本质上是一个带训练参数的loss计算层。
@@ -872,17 +887,54 @@ class MaximumEntropyMarkovModel(Layer):
         return dict(list(base_config.items()) + list(config.items()))
 
 
+class Loss(Layer):
+    """特殊的层，用来定义复杂loss
+    """
+    def __init__(self, output_axis=None, **kwargs):
+        super(Loss, self).__init__(**kwargs)
+        self.output_axis = output_axis
+
+    def call(self, inputs, mask=None):
+        loss = self.compute_loss(inputs, mask)
+        self.add_loss(loss)
+        if self.output_axis is None:
+            return inputs
+        elif isinstance(self.output_axis, list):
+            return [inputs[i] for i in self.output_axis]
+        else:
+            return inputs[self.output_axis]
+
+    def compute_loss(self, inputs, mask=None):
+        raise NotImplementedError
+
+    def compute_output_shape(self, input_shape):
+        if self.output_axis is None:
+            return input_shape
+        elif isinstance(self.output_axis, list):
+            return [input_shape[i] for i in self.output_axis]
+        else:
+            return input_shape[self.output_axis]
+
+    def get_config(self):
+        config = {
+            'output_axis': self.output_axis,
+        }
+        base_config = super(Loss, self).get_config()
+        return dict(list(base_config.items()) + list(config.items()))
+
+
 custom_objects = {
     'Embedding': Embedding,
+    'BiasAdd': BiasAdd,
     'MultiHeadAttention': MultiHeadAttention,
     'LayerNormalization': LayerNormalization,
     'PositionEmbedding': PositionEmbedding,
     'RelativePositionEmbedding': RelativePositionEmbedding,
     'RelativePositionEmbeddingT5': RelativePositionEmbeddingT5,
     'FeedForward': FeedForward,
-    'EmbeddingDense': EmbeddingDense,
     'ConditionalRandomField': ConditionalRandomField,
     'MaximumEntropyMarkovModel': MaximumEntropyMarkovModel,
+    'Loss': Loss,
 }
 
 keras.utils.get_custom_objects().update(custom_objects)
